@@ -1,18 +1,22 @@
+use core::borrow::Borrow;
 use core::iter::FromIterator;
 use core::ops::Index;
 
 use crate::tree::{ConsumingIter, Iter, IterMut, SGTree};
 
+#[cfg(feature = "high_assurance")]
+use crate::tree::SGErr;
+
 /// Ordered map.
 /// A wrapper interface for `SGTree`.
 /// API examples and descriptions are all adapted or directly copied from the standard library's [`BTreeMap`](https://doc.rust-lang.org/std/collections/struct.BTreeMap.html).
-#[allow(clippy::upper_case_acronyms)] // Removal == breaking change, e.g. v2.0
+#[allow(clippy::upper_case_acronyms)] // TODO: Removal == breaking change, e.g. v2.0
 pub struct SGMap<K: Ord, V> {
     bst: SGTree<K, V>,
 }
 
 impl<K: Ord, V> SGMap<K, V> {
-    /// Constructor.
+    /// Makes a new, empty `SGMap`.
     ///
     /// # Examples
     ///
@@ -28,7 +32,7 @@ impl<K: Ord, V> SGMap<K, V> {
     }
 
     /// `#![no_std]`: total capacity, e.g. maximum number of map pairs.
-    /// Attempting to insert pairs beyond capacity will panic.
+    /// Attempting to insert pairs beyond capacity will panic, unless the `high_assurance` feature is enabled.
     ///
     /// If using `std`: fast capacity, e.g. number of map pairs stored on the stack.
     /// Pairs inserted beyond capacity will be stored on the heap.
@@ -74,10 +78,44 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(a[&4], "e");
     /// assert_eq!(a[&5], "f");
     /// ```
+    #[cfg(not(feature = "high_assurance"))]
     pub fn append(&mut self, other: &mut SGMap<K, V>) {
         self.bst.append(&mut other.bst);
     }
 
+    /// Attempts to move all elements from `other` into `self`, leaving `other` empty.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scapegoat::SGMap;
+    ///
+    /// let mut a = SGMap::new();
+    /// a.insert(1, "a");
+    /// a.insert(2, "b");
+    /// a.insert(3, "c");
+    ///
+    /// let mut b = SGMap::new();
+    /// b.insert(3, "d");
+    /// b.insert(4, "e");
+    /// b.insert(5, "f");
+    ///
+    /// a.append(&mut b);
+    ///
+    /// assert_eq!(a.len(), 5);
+    /// assert_eq!(b.len(), 0);
+    ///
+    /// assert_eq!(a[&1], "a");
+    /// assert_eq!(a[&2], "b");
+    /// assert_eq!(a[&3], "d");
+    /// assert_eq!(a[&4], "e");
+    /// assert_eq!(a[&5], "f");
+    /// ```
+    #[cfg(feature = "high_assurance")]
+    pub fn append(&mut self, other: &mut SGMap<K, V>) -> Result<(), SGErr> {
+        self.bst.append(&mut other.bst)
+    }
+
     /// Insert a key-value pair into the map.
     /// If the map did not have this key present, `None` is returned.
     /// If the map did have this key present, the value is updated, the old value is returned,
@@ -96,7 +134,46 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.insert(37, "c"), Some("b"));
     /// assert_eq!(map[&37], "c");
     /// ```
-    pub fn insert(&mut self, key: K, val: V) -> Option<V> {
+    #[cfg(not(feature = "high_assurance"))]
+    pub fn insert(&mut self, key: K, val: V) -> Option<V>
+    where
+        K: Ord,
+    {
+        self.bst.insert(key, val)
+    }
+
+    /// Insert a key-value pair into the map.
+    /// Returns `Err` if map's stack capacity is full, else the `Ok` contains:
+    /// * `None` if the map did not have this key present.
+    /// * The old value if the tree did have this key present (both the value and key are updated,
+    /// this accommodates types that can be `==` without being identical).
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scapegoat::{SGMap, SGErr};
+    ///
+    /// let mut map = SGMap::new();
+    /// assert_eq!(map.insert(37, "a"), Ok(None));
+    /// assert_eq!(map.is_empty(), false);
+    ///
+    /// map.insert(37, "b");
+    /// assert_eq!(map.insert(37, "c"), Ok(Some("b")));
+    /// assert_eq!(map[&37], "c");
+    ///
+    /// let mut key = 38;
+    /// while map.len() < map.capacity() {
+    ///     map.insert(key, "filler");
+    ///     key += 1;
+    /// }
+    ///
+    /// assert_eq!(map.insert(key, "out of bounds"), Err(SGErr::StackCapacityExceeded));
+    /// ```
+    #[cfg(feature = "high_assurance")]
+    pub fn insert(&mut self, key: K, val: V) -> Result<Option<V>, SGErr>
+    where
+        K: Ord,
+    {
         self.bst.insert(key, val)
     }
 
@@ -153,7 +230,11 @@ impl<K: Ord, V> SGMap<K, V> {
         IterMut::new(&mut self.bst)
     }
 
-    /// Removes a key from the map, returning the stored key and value if the key was previously in the map.
+    /// Removes a key from the map, returning the stored key and value if the key
+    /// was previously in the map.
+    ///
+    /// The key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     ///
     /// # Examples
     ///
@@ -165,11 +246,81 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.remove_entry(&1), Some((1, "a")));
     /// assert_eq!(map.remove_entry(&1), None);
     /// ```
-    pub fn remove_entry(&mut self, key: &K) -> Option<(K, V)> {
+    pub fn remove_entry<Q>(&mut self, key: &Q) -> Option<(K, V)>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.remove_entry(key)
     }
 
-    /// Removes a key from the map, returning the value at the key if the key was previously in the map.
+    /// Retains only the elements specified by the predicate.
+    ///
+    /// In other words, remove all pairs `(k, v)` such that `f(&k, &mut v)` returns `false`.
+    /// The elements are visited in ascending key order.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scapegoat::SGMap;
+    ///
+    /// let mut map: SGMap<i32, i32> = (0..8).map(|x| (x, x*10)).collect();
+    /// // Keep only the elements with even-numbered keys.
+    /// map.retain(|&k, _| k % 2 == 0);
+    /// assert!(map.into_iter().eq(vec![(0, 0), (2, 20), (4, 40), (6, 60)]));
+    /// ```
+    pub fn retain<F>(&mut self, mut f: F)
+    where
+        K: Ord,
+        F: FnMut(&K, &mut V) -> bool,
+    {
+        self.bst.retain(|k, v| f(k, v));
+    }
+
+    /// Splits the collection into two at the given key. Returns everything after the given key,
+    /// including the key.
+    ///
+    /// # Examples
+    ///
+    /// Basic usage:
+    ///
+    /// ```
+    /// use scapegoat::SGMap;
+    ///
+    /// let mut a = SGMap::new();
+    /// a.insert(1, "a");
+    /// a.insert(2, "b");
+    /// a.insert(3, "c");
+    /// a.insert(17, "d");
+    /// a.insert(41, "e");
+    ///
+    /// let b = a.split_off(&3);
+    ///
+    /// assert_eq!(a.len(), 2);
+    /// assert_eq!(b.len(), 3);
+    ///
+    /// assert_eq!(a[&1], "a");
+    /// assert_eq!(a[&2], "b");
+    ///
+    /// assert_eq!(b[&3], "c");
+    /// assert_eq!(b[&17], "d");
+    /// assert_eq!(b[&41], "e");
+    /// ```
+    pub fn split_off<Q>(&mut self, key: &Q) -> SGMap<K, V>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
+        SGMap {
+            bst: self.bst.split_off(key),
+        }
+    }
+
+    /// Removes a key from the map, returning the value at the key if the key
+    /// was previously in the map.
+    ///
+    /// The key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     ///
     /// # Examples
     ///
@@ -181,11 +332,18 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.remove(&1), Some("a"));
     /// assert_eq!(map.remove(&1), None);
     /// ```
-    pub fn remove(&mut self, key: &K) -> Option<V> {
+    pub fn remove<Q>(&mut self, key: &Q) -> Option<V>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.remove(key)
     }
 
-    /// Returns the key-value pair corresponding to the given key.
+    /// Returns the key-value pair corresponding to the supplied key.
+    ///
+    /// The supplied key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     ///
     /// # Examples
     ///
@@ -197,11 +355,18 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.get_key_value(&1), Some((&1, &"a")));
     /// assert_eq!(map.get_key_value(&2), None);
     /// ```
-    pub fn get_key_value(&self, key: &K) -> Option<(&K, &V)> {
+    pub fn get_key_value<Q>(&self, key: &Q) -> Option<(&K, &V)>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.get_key_value(key)
     }
 
-    /// Returns a reference to the value corresponding to the given key.
+    /// Returns a reference to the value corresponding to the key.
+    ///
+    /// The key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     ///
     /// # Examples
     ///
@@ -213,11 +378,18 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.get(&1), Some(&"a"));
     /// assert_eq!(map.get(&2), None);
     /// ```
-    pub fn get(&self, key: &K) -> Option<&V> {
+    pub fn get<Q>(&self, key: &Q) -> Option<&V>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.get(key)
     }
 
-    /// Get mutable reference corresponding to key.
+    // Returns a mutable reference to the value corresponding to the key.
+    ///
+    /// The key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     ///
     /// # Examples
     ///
@@ -231,7 +403,11 @@ impl<K: Ord, V> SGMap<K, V> {
     /// }
     /// assert_eq!(map[&1], "b");
     /// ```
-    pub fn get_mut(&mut self, key: &K) -> Option<&mut V> {
+    pub fn get_mut<Q>(&mut self, key: &Q) -> Option<&mut V>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.get_mut(key)
     }
 
@@ -251,8 +427,10 @@ impl<K: Ord, V> SGMap<K, V> {
         self.bst.clear()
     }
 
-    /// Returns `true` if the map contains a value for the given key.
+    /// Returns `true` if the map contains a value for the specified key.
     ///
+    /// The key may be any borrowed form of the map's key type, but the ordering
+    /// on the borrowed form *must* match the ordering on the key type.
     /// # Examples
     ///
     /// ```
@@ -263,7 +441,11 @@ impl<K: Ord, V> SGMap<K, V> {
     /// assert_eq!(map.contains_key(&1), true);
     /// assert_eq!(map.contains_key(&2), false);
     /// ```
-    pub fn contains_key(&self, key: &K) -> bool {
+    pub fn contains_key<Q>(&self, key: &Q) -> bool
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+    {
         self.bst.contains_key(key)
     }
 
@@ -297,7 +479,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// map.insert(2, "a");
     /// assert_eq!(map.first_key_value(), Some((&1, &"b")));
     /// ```
-    pub fn first_key_value(&self) -> Option<(&K, &V)> {
+    pub fn first_key_value(&self) -> Option<(&K, &V)>
+    where
+        K: Ord,
+    {
         self.bst.first_key_value()
     }
 
@@ -314,7 +499,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// map.insert(2, "a");
     /// assert_eq!(map.first_key(), Some(&1));
     /// ```
-    pub fn first_key(&self) -> Option<&K> {
+    pub fn first_key(&self) -> Option<&K>
+    where
+        K: Ord,
+    {
         self.bst.first_key()
     }
 
@@ -336,7 +524,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// }
     /// assert!(map.is_empty());
     /// ```
-    pub fn pop_first(&mut self) -> Option<(K, V)> {
+    pub fn pop_first(&mut self) -> Option<(K, V)>
+    where
+        K: Ord,
+    {
         self.bst.pop_first()
     }
 
@@ -353,7 +544,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// map.insert(2, "a");
     /// assert_eq!(map.last_key_value(), Some((&2, &"a")));
     /// ```
-    pub fn last_key_value(&self) -> Option<(&K, &V)> {
+    pub fn last_key_value(&self) -> Option<(&K, &V)>
+    where
+        K: Ord,
+    {
         self.bst.last_key_value()
     }
 
@@ -369,7 +563,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// map.insert(2, "a");
     /// assert_eq!(map.last_key(), Some(&2));
     /// ```
-    pub fn last_key(&self) -> Option<&K> {
+    pub fn last_key(&self) -> Option<&K>
+    where
+        K: Ord,
+    {
         self.bst.last_key()
     }
 
@@ -391,7 +588,10 @@ impl<K: Ord, V> SGMap<K, V> {
     /// }
     /// assert!(map.is_empty());
     /// ```
-    pub fn pop_last(&mut self) -> Option<(K, V)> {
+    pub fn pop_last(&mut self) -> Option<(K, V)>
+    where
+        K: Ord,
+    {
         self.bst.pop_last()
     }
 
@@ -443,7 +643,11 @@ impl<K: Ord, V> FromIterator<(K, V)> for SGMap<K, V> {
 impl<K: Ord, V> Extend<(K, V)> for SGMap<K, V> {
     fn extend<T: IntoIterator<Item = (K, V)>>(&mut self, iter: T) {
         iter.into_iter().for_each(move |(k, v)| {
+            #[cfg(not(feature = "high_assurance"))]
             self.insert(k, v);
+
+            #[cfg(feature = "high_assurance")]
+            self.insert(k, v).expect("Stack-storage capacity exceeded!");
         });
     }
 
