@@ -1,12 +1,14 @@
-use std::collections::BTreeSet;
-use std::fmt;
+use std::collections::{BTreeMap, BTreeSet};
 use std::iter::FromIterator;
+use std::mem::size_of;
 
 use super::SGTree;
 
 use rand::rngs::SmallRng;
 use rand::{Rng, SeedableRng};
 
+use crate::MAX_ELEMS;
+
 // Test Helpers --------------------------------------------------------------------------------------------------------
 
 // Build a small tree for testing.
@@ -17,7 +19,15 @@ pub fn get_test_tree_and_keys() -> (SGTree<usize, &'static str>, Vec<usize>) {
     assert!(sgt.is_empty());
 
     for k in &keys {
+        #[cfg(not(feature = "high_assurance"))]
         sgt.insert(*k, "n/a");
+
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(*k, "n/a");
+        }
+
         assert_logical_invariants(&sgt);
     }
 
@@ -31,65 +41,6 @@ pub fn get_test_tree_and_keys() -> (SGTree<usize, &'static str>, Vec<usize>) {
     (sgt, keys)
 }
 
-// Pretty print tree.
-pub fn pretty_print<K: Ord + fmt::Debug, V>(sgt: &SGTree<K, V>) {
-    let sgt_lisp = sgt_to_lisp_str(sgt);
-    if sgt_lisp == "()" {
-        println!("(empty tree)");
-    } else {
-        println!(
-            "{}",
-            ruut::prettify(
-                sgt_lisp,
-                ruut::InputFormat::LispLike,
-                "unused".to_string(),
-                "unused".to_string(),
-                None
-            )
-            .unwrap()
-        );
-    }
-}
-
-// Convert tree to a Lisp-like string.
-fn sgt_to_lisp_str<K: Ord + fmt::Debug, V>(sgt: &SGTree<K, V>) -> String {
-    match sgt.root_idx {
-        Some(idx) => sgt_to_lisp_str_helper(sgt, idx),
-        None => String::from("()"),
-    }
-}
-
-// Helper function to convert tree to a Lisp-like string.
-fn sgt_to_lisp_str_helper<K: Ord + fmt::Debug, V>(sgt: &SGTree<K, V>, idx: usize) -> String {
-    let node = sgt.arena.hard_get(idx);
-    match (node.left_idx, node.right_idx) {
-        // No children
-        (None, None) => format!("{:?} [{}]", node.key, idx),
-        // Left child only
-        (Some(left_idx), None) => format!(
-            "{:?} [{}] ({})",
-            node.key,
-            idx,
-            sgt_to_lisp_str_helper(sgt, left_idx)
-        ),
-        // Right child only
-        (None, Some(right_idx)) => format!(
-            "{:?} [{}] ({})",
-            node.key,
-            idx,
-            sgt_to_lisp_str_helper(sgt, right_idx)
-        ),
-        // Two children
-        (Some(left_idx), Some(right_idx)) => format!(
-            "{:?} [{}] ({}, {})",
-            node.key,
-            idx,
-            sgt_to_lisp_str_helper(sgt, left_idx),
-            sgt_to_lisp_str_helper(sgt, right_idx)
-        ),
-    }
-}
-
 // Verify three logical invariants for the tree:
 // 1. A right child node's key is always greater than it's parent's key.
 // 2. A left child node's key is always less than it's parent's key.
@@ -137,7 +88,6 @@ fn logical_fuzz(iter_cnt: usize, check_invars: bool) {
     let mut shadow_keys = BTreeSet::new();
     let mut fast_rng = SmallRng::from_entropy();
     let mut slow_rng = rand::thread_rng();
-    let mut removal_cnt = 0;
 
     for _ in 0..iter_cnt {
         let rand_key: usize;
@@ -148,9 +98,17 @@ fn logical_fuzz(iter_cnt: usize, check_invars: bool) {
         }
 
         // Rand value insert
-        sgt.insert(rand_key, "n/a");
         shadow_keys.insert(rand_key);
 
+        #[cfg(not(feature = "high_assurance"))]
+        sgt.insert(rand_key, "n/a");
+
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(rand_key, "n/a");
+        }
+
         // Verify internal state post-insert
         if check_invars {
             assert_logical_invariants(&sgt);
@@ -162,7 +120,6 @@ fn logical_fuzz(iter_cnt: usize, check_invars: bool) {
             assert!(shadow_keys.remove(&rand_key));
             assert!(sgt.contains_key(&rand_key));
             sgt.remove(&rand_key);
-            removal_cnt += 1;
 
             // Verify internal state post-remove
             if check_invars {
@@ -171,7 +128,6 @@ fn logical_fuzz(iter_cnt: usize, check_invars: bool) {
         }
     }
 
-    let rebal_cnt = sgt.rebal_cnt();
     let final_keys = sgt.into_iter().map(|(k, _)| k).collect::<BTreeSet<usize>>();
 
     if final_keys != shadow_keys {
@@ -184,24 +140,35 @@ fn logical_fuzz(iter_cnt: usize, check_invars: bool) {
         );
         panic!("Keys do not match shadow set!");
     }
-
-    println!(
-        "Fuzz summary: {} iterations, {} removals, {} rebalances.",
-        iter_cnt, removal_cnt, rebal_cnt
-    );
 }
 
 // Tests ---------------------------------------------------------------------------------------------------------------
 
+#[test]
+fn test_tree_packing() {
+    // Assumes `SG_MAX_STACK_ELEMS == 1024` (default)
+    if MAX_ELEMS == 1024 {
+        #[cfg(target_pointer_width = "64")]
+        #[cfg(not(feature = "high_assurance"))]
+        {
+            assert_eq!(size_of::<SGTree<u32, u32>>(), 49_240);
+        }
+
+        #[cfg(target_pointer_width = "64")]
+        #[cfg(feature = "high_assurance")]
+        {
+            assert_eq!(size_of::<SGTree<u32, u32>>(), 18_488);
+        }
+    }
+}
+
 #[test]
 fn test_ref_iter() {
     let (sgt, keys) = get_test_tree_and_keys();
     let mut ref_iter_keys = Vec::<usize>::new();
 
-    println!("\nReference iterator output:\n");
     for (k, _) in &sgt {
         ref_iter_keys.push(*k);
-        println!("ITER: {}", k);
     }
 
     let k_1 = BTreeSet::from_iter(keys.iter().cloned());
@@ -215,10 +182,8 @@ fn test_iter() {
     let (sgt, keys) = get_test_tree_and_keys();
     let mut iter_keys = Vec::<usize>::new();
 
-    println!("\nConsuming iterator output:\n");
     for (k, _) in sgt {
         iter_keys.push(k);
-        println!("ITER: {}", k);
     }
 
     let k_1 = BTreeSet::from_iter(keys.iter().cloned());
@@ -243,19 +208,52 @@ fn test_from_iter() {
     );
 }
 
+#[cfg(feature = "high_assurance")]
+#[should_panic(expected = "Stack-storage capacity exceeded!")]
+#[test]
+fn test_from_iter_panic() {
+    let sgt_temp: SGTree<isize, isize> = SGTree::new();
+    let max_capacity = sgt_temp.capacity();
+    let _ = SGTree::from_iter((0..(max_capacity + 1)).map(|val| (val, val)));
+}
+
 #[test]
 fn test_append() {
     let mut a = SGTree::new();
-    a.insert(1, "1");
-    a.insert(2, "2");
-    a.insert(3, "3");
+
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        a.insert(1, "1");
+        a.insert(2, "2");
+        a.insert(3, "3");
+    }
+
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        a.insert(1, "1");
+        a.insert(2, "2");
+        a.insert(3, "3");
+    }
 
     let mut b = SGTree::new();
-    b.insert(4, "4");
-    b.insert(5, "5");
-    b.insert(6, "6");
 
-    a.append(&mut b);
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        b.insert(4, "4");
+        b.insert(5, "5");
+        b.insert(6, "6");
+        a.append(&mut b);
+    }
+
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        b.insert(4, "4");
+        b.insert(5, "5");
+        b.insert(6, "6");
+        a.append(&mut b);
+    }
 
     assert!(b.is_empty());
     assert_eq!(a.len(), 6);
@@ -273,21 +271,19 @@ fn test_two_child_removal_case_1() {
     let to_remove = 2;
 
     for k in &keys {
+        #[cfg(not(feature = "high_assurance"))]
         sgt.insert(*k, "n/a");
-    }
 
-    println!("\nBefore two child removal case 1:\n");
-    pretty_print(&sgt);
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(*k, "n/a");
+        }
+    }
 
     sgt.remove(&to_remove);
     assert_logical_invariants(&sgt);
 
-    println!(
-        "\nAfter two child removal case 1 (removed {}):\n",
-        to_remove
-    );
-    pretty_print(&sgt);
-
     assert_eq!(
         sgt.into_iter().map(|(k, _)| k).collect::<Vec<usize>>(),
         vec![1, 3]
@@ -301,21 +297,19 @@ fn test_two_child_removal_case_2() {
     let to_remove = 2;
 
     for k in &keys {
+        #[cfg(not(feature = "high_assurance"))]
         sgt.insert(*k, "n/a");
-    }
 
-    println!("\nBefore two child removal case 2:\n");
-    pretty_print(&sgt);
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(*k, "n/a");
+        }
+    }
 
     sgt.remove(&to_remove);
     assert_logical_invariants(&sgt);
 
-    println!(
-        "\nAfter two child removal case 2 (removed {}):\n",
-        to_remove
-    );
-    pretty_print(&sgt);
-
     assert_eq!(
         sgt.into_iter().map(|(k, _)| k).collect::<Vec<usize>>(),
         vec![1, 3, 4]
@@ -329,21 +323,19 @@ fn test_two_child_removal_case_3() {
     let to_remove = 3;
 
     for k in &keys {
+        #[cfg(not(feature = "high_assurance"))]
         sgt.insert(*k, "n/a");
-    }
 
-    println!("\nBefore two child removal case 3:\n");
-    pretty_print(&sgt);
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(*k, "n/a");
+        }
+    }
 
     sgt.remove(&to_remove);
     assert_logical_invariants(&sgt);
 
-    println!(
-        "\nAfter two child removal case 3 (removed {}):\n",
-        to_remove
-    );
-    pretty_print(&sgt);
-
     assert_eq!(
         sgt.into_iter().map(|(k, _)| k).collect::<Vec<usize>>(),
         vec![1, 2, 4, 5, 6]
@@ -355,32 +347,17 @@ fn test_rand_remove() {
     let (mut sgt, mut keys) = get_test_tree_and_keys();
     let mut rng = SmallRng::from_entropy();
 
-    println!(
-        "\nAfter {} insertions, {} rebalance(s):\n",
-        keys.len(),
-        sgt.rebal_cnt()
-    );
-    pretty_print(&sgt);
-
     // Remove half of keys at random
     let mut keys_to_remove = Vec::new();
     for _ in 0..=(keys.len() / 2) {
         keys_to_remove.push(keys.remove(rng.gen_range(0, keys.len())));
     }
     for k in &keys_to_remove {
-        println!("Removing {}", k);
         assert!(sgt.contains_key(k));
         let (removed_key, _) = sgt.remove_entry(k).unwrap();
         assert_eq!(*k, removed_key);
         assert_logical_invariants(&sgt);
     }
-
-    println!(
-        "\nAfter {} insertions, {} rebalance(s):\n",
-        keys_to_remove.len(),
-        sgt.rebal_cnt()
-    );
-    pretty_print(&sgt);
 }
 
 #[test]
@@ -420,7 +397,15 @@ fn test_first_last() {
     let keys = vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
     let mut sgt = SGTree::new();
     for k in &keys {
+        #[cfg(not(feature = "high_assurance"))]
         sgt.insert(*k, "n/a");
+
+        #[allow(unused_must_use)]
+        #[cfg(feature = "high_assurance")]
+        {
+            sgt.insert(*k, "n/a");
+        }
+
         assert_logical_invariants(&sgt);
         sgt.contains_key(k);
     }
@@ -443,28 +428,63 @@ fn test_first_last() {
 fn test_subtree_rebalance() {
     let mut sgt: SGTree<usize, &str> = SGTree::new();
 
-    sgt.insert(237197427728999687, "n/a");
-    sgt.insert(2328219650045037451, "n/a");
-    sgt.insert(13658362701324851025, "n/a");
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        sgt.insert(237197427728999687, "n/a");
+        sgt.insert(2328219650045037451, "n/a");
+        sgt.insert(13658362701324851025, "n/a");
+    }
+
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        sgt.insert(237197427728999687, "n/a");
+        sgt.insert(2328219650045037451, "n/a");
+        sgt.insert(13658362701324851025, "n/a");
+    }
 
     sgt.remove(&13658362701324851025);
 
-    sgt.insert(2239831466376212988, "n/a");
-    sgt.insert(15954331640746224573, "n/a");
-    sgt.insert(8202281457156668544, "n/a");
-    sgt.insert(5226917524540172628, "n/a");
-    sgt.insert(11823668523937575827, "n/a");
-    sgt.insert(13519144312507908668, "n/a");
-    sgt.insert(17799627035639903362, "n/a");
-    sgt.insert(17491737414383996868, "n/a");
-    sgt.insert(2247619647701733096, "n/a");
-    sgt.insert(15122725631405182851, "n/a");
-    sgt.insert(9837932133859010449, "n/a");
-    sgt.insert(15426779056379992972, "n/a");
-    sgt.insert(1963900452029117196, "n/a");
-    sgt.insert(1328762018325194497, "n/a");
-    sgt.insert(7471075696232724572, "n/a");
-    sgt.insert(9350363297060113585, "n/a");
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        sgt.insert(2239831466376212988, "n/a");
+        sgt.insert(15954331640746224573, "n/a");
+        sgt.insert(8202281457156668544, "n/a");
+        sgt.insert(5226917524540172628, "n/a");
+        sgt.insert(11823668523937575827, "n/a");
+        sgt.insert(13519144312507908668, "n/a");
+        sgt.insert(17799627035639903362, "n/a");
+        sgt.insert(17491737414383996868, "n/a");
+        sgt.insert(2247619647701733096, "n/a");
+        sgt.insert(15122725631405182851, "n/a");
+        sgt.insert(9837932133859010449, "n/a");
+        sgt.insert(15426779056379992972, "n/a");
+        sgt.insert(1963900452029117196, "n/a");
+        sgt.insert(1328762018325194497, "n/a");
+        sgt.insert(7471075696232724572, "n/a");
+        sgt.insert(9350363297060113585, "n/a");
+    }
+
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        sgt.insert(2239831466376212988, "n/a");
+        sgt.insert(15954331640746224573, "n/a");
+        sgt.insert(8202281457156668544, "n/a");
+        sgt.insert(5226917524540172628, "n/a");
+        sgt.insert(11823668523937575827, "n/a");
+        sgt.insert(13519144312507908668, "n/a");
+        sgt.insert(17799627035639903362, "n/a");
+        sgt.insert(17491737414383996868, "n/a");
+        sgt.insert(2247619647701733096, "n/a");
+        sgt.insert(15122725631405182851, "n/a");
+        sgt.insert(9837932133859010449, "n/a");
+        sgt.insert(15426779056379992972, "n/a");
+        sgt.insert(1963900452029117196, "n/a");
+        sgt.insert(1328762018325194497, "n/a");
+        sgt.insert(7471075696232724572, "n/a");
+        sgt.insert(9350363297060113585, "n/a");
+    }
 
     sgt.remove(&9350363297060113585);
 
@@ -472,13 +492,14 @@ fn test_subtree_rebalance() {
     assert!(sgt.contains_key(&13519144312507908668));
     let critical_val = 11827258012878092103;
 
-    println!("\nBefore inserting {}:\n", critical_val);
-    pretty_print(&sgt);
-
+    #[cfg(not(feature = "high_assurance"))]
     sgt.insert(critical_val, "n/a");
 
-    println!("\nAfter inserting {}:\n", critical_val);
-    pretty_print(&sgt);
+    #[cfg(feature = "high_assurance")]
+    #[allow(unused_must_use)]
+    {
+        sgt.insert(critical_val, "n/a");
+    }
 
     assert!(sgt.contains_key(&11823668523937575827));
     assert!(sgt.contains_key(&critical_val));
@@ -489,6 +510,8 @@ fn test_subtree_rebalance() {
 fn test_logical_fuzz_fast() {
     let sgt: SGTree<usize, &str> = SGTree::new();
     logical_fuzz(sgt.capacity(), false); // Stack-only
+
+    #[cfg(not(feature = "high_assurance"))]
     logical_fuzz(sgt.capacity() + 2_000, false); // Stack + Heap
 }
 
@@ -496,5 +519,66 @@ fn test_logical_fuzz_fast() {
 fn test_logical_fuzz_slow() {
     let sgt: SGTree<usize, &str> = SGTree::new();
     logical_fuzz(sgt.capacity(), true); // Stack-only
+
+    #[cfg(not(feature = "high_assurance"))]
     logical_fuzz(sgt.capacity() + 2_000, true); // Stack + Heap
 }
+
+#[test]
+fn test_retain() {
+    let mut bt_map: BTreeMap<usize, usize> = BTreeMap::new();
+    bt_map.insert(14987934384537018497, 0);
+    bt_map.insert(14483576400934207487, 0);
+
+    let mut sg_map: SGTree<usize, usize> = SGTree::new();
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        sg_map.insert(14987934384537018497, 0);
+        sg_map.insert(14483576400934207487, 0);
+    }
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        sg_map.insert(14987934384537018497, 0);
+        sg_map.insert(14483576400934207487, 0);
+    }
+
+    assert!(sg_map.iter().eq(bt_map.iter()));
+
+    sg_map.retain(|&k, _| (k % 16766697) % 2 == 0);
+    bt_map.retain(|&k, _| (k % 16766697) % 2 == 0);
+
+    assert!(sg_map.iter().eq(bt_map.iter()));
+}
+
+#[test]
+fn test_slice_search() {
+    let bad_code: [u8; 8] = [0xB, 0xA, 0xA, 0xD, 0xC, 0x0, 0xD, 0xE];
+    let bad_food: [u8; 8] = [0xB, 0xA, 0xA, 0xD, 0xF, 0x0, 0x0, 0xD];
+
+    assert_eq!(std::mem::size_of_val(&bad_code), 8);
+    assert_eq!(std::mem::size_of_val(&bad_food), 8);
+
+    let mut sgt = SGTree::new();
+    #[cfg(not(feature = "high_assurance"))]
+    {
+        sgt.insert(bad_code, "badcode");
+        sgt.insert(bad_food, "badfood");
+    }
+    #[allow(unused_must_use)]
+    #[cfg(feature = "high_assurance")]
+    {
+        sgt.insert(bad_code, "badcode");
+        sgt.insert(bad_food, "badfood");
+    }
+
+    let bad_vec: Vec<u8> = vec![0xB, 0xA, 0xA, 0xD];
+    let bad_food_vec: Vec<u8> = vec![0xB, 0xA, 0xA, 0xD, 0xF, 0x0, 0x0, 0xD];
+    let bad_dude_vec: Vec<u8> = vec![0xB, 0xA, 0xA, 0xD, 0xD, 0x0, 0x0, 0xD];
+
+    assert_eq!(sgt.get(&bad_food_vec[..]), Some(&"badfood"));
+
+    assert_eq!(sgt.get(&bad_vec[..]), None);
+
+    assert_eq!(sgt.get(&bad_dude_vec[..]), None);
+}
